// import * as THREE from "three";
// import { GLTFLoader } from "three/examples/jsm/loaders/GLTFLoader";
// import * as ZapparThree from "@zappar/zappar-threejs";

declare var THREE: any;
let GLTFLoader = THREE.GLTFLoader;
declare var ZapparThree: any;

import "./style.css";

// ZapparThree provides a LoadingManager that shows a progress bar while
// the assets are downloaded
let manager = new ZapparThree.LoadingManager();

// Setup ThreeJS in the usual way
let renderer = new THREE.WebGLRenderer();
document.body.appendChild(renderer.domElement);

renderer.shadowMap.enabled = true;
renderer.shadowMap.type = THREE.PCFSoftShadowMap;
renderer.outputEncoding = THREE.sRGBEncoding;

renderer.setSize(window.innerWidth, window.innerHeight);
window.addEventListener("resize", () => {
    renderer.setSize(window.innerWidth, window.innerHeight);
});

// Setup a Zappar camera instead of one of ThreeJS's cameras
let camera = new ZapparThree.Camera();
camera.backgroundTexture.encoding = THREE.sRGBEncoding;

// The Zappar library needs your WebGL context, so pass it
ZapparThree.glContextSet(renderer.getContext());

// Create a ThreeJS Scene and set its background to be the camera background texture
let scene = new THREE.Scene();
scene.background = camera.backgroundTexture;

// Request the necessary permission from the user
ZapparThree.permissionRequestUI().then(function (granted: boolean) {
    if (granted) camera.start();
    else ZapparThree.permissionDeniedUI();
});

// Set up our instant tracker group
let tracker = new ZapparThree.InstantWorldTracker();
let trackerGroup: any = new ZapparThree.InstantWorldAnchorGroup(camera, tracker);
scene.add(trackerGroup);

// let env = new ZapparThree.CameraEnvironmentMap();
// scene.environment = env.environmentMap;

let light = new THREE.DirectionalLight();
light.target = trackerGroup;
light.castShadow = true;

light.position.set(0, 1, 0);
trackerGroup.add(light);

let ambient = new THREE.AmbientLight("white", 0.03);
trackerGroup.add(ambient);
// Add some content
let box = new THREE.Mesh(
    new THREE.PlaneBufferGeometry(),
    new THREE.MeshBasicMaterial({
        map: new THREE.TextureLoader().load(require("file-loader!./hotspot.png").default),
        transparent: true
    })
);
box.position.set(0, 0, 0);
box.rotateX(-0.5 * Math.PI);
trackerGroup.add(box);

let placementTime = 0;

let mixer: any;

let playAnimation : undefined | (() => void);
let pickUp : undefined | (() => void);

new GLTFLoader().load(require("file-loader!./RobotExpressive.glb").default, (gltf: any) => {
    trackerGroup.add(gltf.scene);
    gltf.scene.scale.set(0.3, 0.3, 0.3);
    console.log(gltf.animations);

    gltf.scene.traverse((node: any) => {
        node.frustumCulled = false;
        if (node instanceof THREE.Mesh) {
            node.castShadow = true;
            node.receiveShadow = true;
        }
    });
        


    mixer = new THREE.AnimationMixer(gltf.scene);
    let clip = THREE.AnimationClip.findByName(gltf.animations, 'Jump');
    console.log(clip);
    let action = mixer.clipAction(clip);
    action.play();
    action.time = 0.3;
    action.paused = true;
    // action.loop = LoopOnce;

    clip = THREE.AnimationClip.findByName(gltf.animations, 'Idle');
    let idleAction = mixer.clipAction(clip);

    clip = THREE.AnimationClip.findByName(gltf.animations, 'Wave');
    let waveAction = mixer.clipAction(clip);
    
    playAnimation = async () => {
        if (!hasPlaced) return;
        action.paused = false;
        action.fadeOut(0.5);
        idleAction.reset().setEffectiveTimeScale( 1 ).setEffectiveWeight( 1 ).fadeIn( 0.5 ).play();
        let pt = placementTime;

        while(true) {
            if (pt !== placementTime) return;
            await delay(3000 + Math.random() * 3000);
            if (pt !== placementTime) return;
            if (!hasPlaced) return;
            idleAction.fadeOut(0.5);
            waveAction.reset().setEffectiveTimeScale( 1 ).setEffectiveWeight( 1 ).fadeIn( 0.5 ).play();

            await delay(1500);
            if (pt !== placementTime) return;
            if (!hasPlaced) return;
            waveAction.fadeOut(0.5);
            idleAction.reset().setEffectiveTimeScale( 1 ).setEffectiveWeight( 1 ).fadeIn( 0.5 ).play();
        }
    }

    pickUp = async () => {
        waveAction.fadeOut(0.3);
        idleAction.fadeOut(0.3);
        action.reset().setEffectiveTimeScale(1).play();
        let pt = placementTime;
        await delay(300);
        if (pt !== placementTime) return;
        action.paused = true;
    }

})

function delay(ms: number) : Promise<void> {
    return new Promise<void>(resolve => setTimeout(resolve, ms));
}
let shadowPlane = new THREE.Mesh(
    new THREE.PlaneBufferGeometry(),
    new THREE.ShadowMaterial()
);
shadowPlane.rotateX(-1 * Math.PI / 2); // Rotate the plane to be flat on the ground
shadowPlane.material.opacity = 0.3; // Make the plane semi-transparent so some of the ground is visible under the shadow
shadowPlane.receiveShadow = true;
shadowPlane.material.depthWrite = false;
trackerGroup.add(shadowPlane);

let hasPlaced = false;
let placementUI = document.getElementById("zappar-placement-ui") || document.createElement("div");
placementUI.addEventListener("click", () => {

    if (hasPlaced) {
        hasPlaced = false;
        placementUI.innerText = "Place";
        box.material.opacity = 1;
        placementTime++;
        pickUp?.();
        return;
    }
    // placementUI.remove();
    placementUI.innerText = "Pick Up";
    hasPlaced = true;
    playAnimation?.();

    box.material.opacity = 0;
})

const clock = new THREE.Clock();

// Set up our render loop
function render() {
    requestAnimationFrame(render);
    camera.updateFrame(renderer);
    // env.update(renderer, camera);
    mixer?.update(clock.getDelta());

    if (!hasPlaced) tracker.setAnchorPoseFromCameraOffset(0, -0.5, -2.5);

    renderer.render(scene, camera);
}

requestAnimationFrame(render);